@@ -4,6 +4,7 @@
 -- Copyright   :  (c) 2007 Bjorn Bringert,
 --                    2008 Andrea Vezzosi,
 --                    2008-2009 Duncan Coutts
+--                    2011 Max Bolingbroke
 -- License     :  BSD3
 --
 -- Maintainer  :  duncan@community.haskell.org
@@ -48,6 +49,7 @@ module Codec.Archive.Tar.Types (
 
   Entries(..),
   mapEntries,
+  mapEntriesNoFail,
   foldEntries,
   unfoldEntries,
 
@@ -118,6 +120,7 @@ data EntryContent = NormalFile      ByteString !FileSize
                   | BlockDevice     !DevMajor !DevMinor
                   | NamedPipe
                   | OtherEntryType  !TypeCode ByteString !FileSize
+    deriving (Eq, Ord)
 
 data Ownership = Ownership {
     -- | The owner user name. Should be set to @\"\"@ if unknown.
@@ -132,6 +135,7 @@ data Ownership = Ownership {
     -- | Numeric owner group id. Should be set to @0@ if unknown.
     groupId :: !Int
   }
+    deriving (Eq, Ord)
 
 -- | There have been a number of extensions to the tar file format over the
 -- years. They all share the basic entry fields and put more meta-data in
@@ -154,7 +158,7 @@ data Format =
      -- archives the standard USTAR/POSIX should be used.
      --
    | GnuFormat
-  deriving Eq
+  deriving (Eq, Ord)
 
 -- | @rw-r--r--@ for normal files
 ordinaryFilePermissions :: Permissions
@@ -410,9 +414,9 @@ fromLinkTargetToWindowsPath (LinkTarget path) = adjustDirectory $
 -- The 'Monoid' instance lets you concatenate archives or append entries to an
 -- archive.
 --
-data Entries = Next Entry Entries
-             | Done
-             | Fail String
+data Entries e = Next Entry (Entries e)
+               | Done
+               | Fail e
 
 -- | This is like the standard 'unfoldr' function on lists, but for 'Entries'.
 -- It includes failure as an extra possibility that the stepper function may
@@ -421,7 +425,7 @@ data Entries = Next Entry Entries
 -- It can be used to generate 'Entries' from some other type. For example it is
 -- used internally to lazily unfold entries from a 'ByteString'.
 --
-unfoldEntries :: (a -> Either String (Maybe (Entry, a))) -> a -> Entries
+unfoldEntries :: (a -> Either e (Maybe (Entry, a))) -> a -> Entries e
 unfoldEntries f = unfold
   where
     unfold x = case f x of
@@ -436,7 +440,7 @@ unfoldEntries f = unfold
 -- This is used to consume a sequence of entries. For example it could be used
 -- to scan a tarball for problems or to collect an index of the contents.
 --
-foldEntries :: (Entry -> a -> a) -> a -> (String -> a) -> Entries -> a
+foldEntries :: (Entry -> a -> a) -> a -> (e -> a) -> Entries e -> a
 foldEntries next done fail' = fold
   where
     fold (Next e es) = next e (fold es)
@@ -446,10 +450,18 @@ foldEntries next done fail' = fold
 -- | This is like the standard 'map' function on lists, but for 'Entries'. It
 -- includes failure as a extra possible outcome of the mapping function.
 --
-mapEntries :: (Entry -> Either String Entry) -> Entries -> Entries
+-- If your mapping function cannot fail it may be more convenient to use
+-- 'mapEntriesNoFail'
+mapEntries :: (Entry -> Either e' Entry) -> Entries e -> Entries (Either e e')
 mapEntries f =
-  foldEntries (\entry rest -> either Fail (flip Next rest) (f entry)) Done Fail
+  foldEntries (\entry rest -> either (Fail . Right) (flip Next rest) (f entry)) Done (Fail . Left)
 
-instance Monoid Entries where
+-- | Like 'mapEntries' but the mapping function itself cannot fail.
+--
+mapEntriesNoFail :: (Entry -> Entry) -> Entries e -> Entries e
+mapEntriesNoFail f =
+  foldEntries (\entry -> Next (f entry)) Done Fail
+
+instance Monoid (Entries e) where
   mempty      = Done
   mappend a b = foldEntries Next b Fail a
